

/*
 *  Author: Arvin Schnell
 */


#include <string.h>
#include <stdlib.h>
#ifdef __NetBSD__
#  include <machine/byte_swap.h>
#else
#  include <byteswap.h>
#endif
#include <iostream>

using std::cerr;

#include "pcm.h"
#include "pcm-oss.h"
#ifdef HAVE_ALSA
#  include "pcm-alsa.h"
#endif


PCM*
PCM::create (const char* name, pcm_mode_t mode, pcm_format_t format,
	     int channels, int rate, int buffer_size)
{
    const char* tmp = name;

#ifdef HAVE_ALSA
    if (tmp == 0 || strcmp (tmp, "default") == 0)
	tmp = "plughw:0,0";
#else

#  ifndef __NetBSD__
    if (tmp == 0 || strcmp (tmp, "default") == 0)
	tmp = "/dev/dsp";
#  else
    if (tmp == 0 || strcmp (tmp, "default") == 0)
	tmp = "/dev/audio";
#  endif

#endif

    if (strncmp (tmp, "/dev/", 5) == 0) {
	PCM* pcm = new OSS (tmp, mode, format, channels, rate, buffer_size);
	if (!pcm->ok) {
	    delete pcm;
	    pcm = 0;
	}
	return pcm;
    }

#ifdef HAVE_ALSA
    if (strncmp (tmp, "plughw:", 7) == 0) {
	PCM* pcm = new ALSA (tmp, mode, format, channels, rate, buffer_size);
	if (!pcm->ok) {
	    delete pcm;
	    pcm = 0;
	}
	return pcm;
    }
#endif

    cerr << "error: unknown name for pcm interface\n";
    return 0;
}


PCM::PCM (const char* name, pcm_mode_t mode, pcm_format_t format,
	  int channels, int rate, int buffer_size)
    : name (strdup (name)),
      mode (mode),
      format (format),
      channels (channels),
      rate (rate),
      buffer_size (buffer_size)
{
    ok = false;
}


PCM::~PCM ()
{
    free (name);
}


void
PCM::info ()
{
    cerr << "name: " << name << '\n'
	 << "mode: " << (mode == PLAY ? "play" : "record") << '\n'
	 << "format: signed " << (8 << format) << " bits\n"
	 << "channels: " << channels << '\n'
	 << "rate: " << rate << " Hz\n"
	 << "buffer size: " << buffer_size << " frames, "
	 << buffer_size * get_bytes_per_frame () << " bytes, "
	 << 1000.0 * buffer_size / rate << " ms\n";
}


void
PCM::swap_buffer (void* buffer, int frames)
{
    swap_buffer (buffer, frames, format, channels);
}


void
PCM::swap_buffer (void* buffer, int frames, pcm_format_t format, int channels)
{
    int n = frames * channels;

    switch (format) {

	case S8: {
	} break;

	case S16: {
	    int16_t* tmp_buffer = (int16_t*) buffer;
	    for (int i = n; i != 0; i--) {
#ifdef __NetBSD__
		*tmp_buffer = bswap16 (*tmp_buffer);
#else
		*tmp_buffer = bswap_16 (*tmp_buffer);
#endif
		tmp_buffer++;
	    }
	} break;

	case S32: {
	    int32_t* tmp_buffer = (int32_t*) buffer;
	    for (int i = n; i != 0; i--) {
#ifdef __NetBSD__
		*tmp_buffer = bswap32 (*tmp_buffer);
#else
		*tmp_buffer = bswap_32 (*tmp_buffer);
#endif
		tmp_buffer++;
	    }
	} break;

    }
}
